class DisableMigrations(object):

    def __init__(self):
        self._django_version = __import__('django').VERSION

    def __contains__(self, item):
        return True

    def __getitem__(self, item):
        if self._django_version >= (1, 9):
            return None
        else:
            return 'notmigrations'



from django.contrib.auth.models import User
from django.core.management import call_command
from django.test import TestCase


class SmokeTests(TestCase):

    @classmethod
    def setUpTestData(cls):
        fixture_path = getattr(cls, 'fixture_path', None)
        if fixture_path:
            call_command('loaddata', fixture_path)

    @classmethod
    def setUpClass(cls):
        super(SmokeTests, cls).setUpClass()
        cls.smoke_user_credentials = {
            'username': 'smoke_superuser',
            'email': 'smoke@test.com',
            'password': 'smoke_password'
        }
        cls.smoke_user = User.objects.create_superuser(
            cls.smoke_user_credentials['username'],
            cls.smoke_user_credentials['email'],
            cls.smoke_user_credentials['password'],
        )

    def setUp(self):
        super(SmokeTests, self).setUp()
        try:
            self.client.force_login(self.smoke_user)  # faster than regular logging
        except AttributeError:
            # force_login available from Django 1.9
            self.client.login(
                usernme=self.smoke_user_credentials['username'],
                password=self.smoke_user_credentials['password']
            )

    def fail_test(self, url, http_method, response):
        fail_msg = (
            '\nSMOKE TEST FAILED'
            '\nURL: {}'
            '\nHTTP METHOD: {}'
            '\nSTATUS CODE: {}'
        ).format(url, http_method, response.status_code)
        self.fail(fail_msg)



import argparse
import os

from django.core.management import BaseCommand, CommandParser
from django.core.management.base import CommandError

from ...generator import SmokeTestsGenerator


class Command(BaseCommand):
    help = "Smoke tests for Django endpoints."

    def create_parser(self, prog_name, subcommand):
        """
        Override in order to skip default parameters like verbosity, version, etc.
        """
        parser = CommandParser(
            self, prog="%s %s" % (os.path.basename(prog_name), subcommand),
            description=self.help or None,
        )
        # create hidden options (required by BaseCommand)
        parser.add_argument('--no-color', help=argparse.SUPPRESS)
        parser.add_argument('--pythonpath', help=argparse.SUPPRESS)
        parser.add_argument('--traceback', help=argparse.SUPPRESS)
        self.add_arguments(parser)
        return parser

    def add_arguments(self, parser):
        methods_group = parser.add_mutually_exclusive_group()
        methods_group.add_argument(
            '--http-methods',
            default=None,
            type=str,
            help='comma separated HTTP methods that will be executed for all endpoints, '
                 'eg. GET,POST,DELETE [default: GET,POST,PUT,DELETE]'
        )
        methods_group.add_argument(
            '-g', '--get-only',
            action='store_true',
            default=False,
            dest='get_only',
            help='shortcut for --http-methods GET'
        )
        parser.add_argument(
            '--allow-status-codes',
            default=None,
            type=str,
            help='comma separated HTTP status codes that will be considered as success responses, '
                 'eg. 200,201,204 [default: 200,201,301,302,304,405]'
        )
        parser.add_argument(
            '--disallow-status-codes',
            default=None,
            type=str,
            help='comma separated HTTP status codes that will be considered as fail responses, '
                 'eg. 404,500'
        )
        parser.add_argument(
            '--settings',
            help=(
                'path to the Django settings module, eg. myproject.settings'
            ),
        )
        parser.add_argument(
            '--configuration',
            help=(
                'name of the configuration class to load, e.g. Development'
            ),
        )
        parser.add_argument(
            '--fixture',
            help=(
                'Django fixture JSON file to be loaded before executing smoke tests'
            ),
        )
        parser.add_argument(
            '--no-migrations',
            dest='no_migrations',
            action='store_true',
            help='flag for skipping migrations, database will be created directly from models'
        )
        parser.set_defaults(no_migrations=False)
        parser.add_argument(
            '--no-db',
            dest='no_db',
            action='store_true',
            help='flag for skipping database creation'
        )
        parser.set_defaults(no_db=False)
        parser.add_argument(
            'app_names',
            default=None,
            nargs='?',
            help='names of apps to test',
        )

    def handle(self, *args, **options):
        if options.get('get_only'):
            methods_to_test = ['GET']
        else:
            methods_to_test = self._get_list_from_string(options.get('http_methods'))
        allowed_status_codes = self._get_list_from_string(options.get('allow_status_codes'))
        disallowed_status_codes = self._get_list_from_string(options.get('disallow_status_codes'))
        disable_migrations = options.get('no_migrations')
        use_db = not options.get('no_db')
        app_names = self._get_list_from_string(options.get('app_names'))
        settings_module = options.get('settings')
        configuration = options.get('configuration')
        fixture_path = options.get('fixture')

        if allowed_status_codes and disallowed_status_codes:
            raise CommandError(
                'You can either specify --allow-status-codes or --disallow-status-codes. '
                'You must not specify both.'
            )

        generator = SmokeTestsGenerator(
            http_methods=methods_to_test,
            allowed_status_codes=allowed_status_codes,
            disallowed_status_codes=disallowed_status_codes,
            use_db=use_db,
            app_names=app_names,
            disable_migrations=disable_migrations,
            settings_module=settings_module,
            configuration=configuration,
            fixture_path=fixture_path,
        )
        generator.execute()

        if generator.warnings:
            self.stdout.write(
                'Some tests were skipped. Please report on '
                'https://github.com/kamilkijak/django-smoke-tests/issues.'
            )
            self.stdout.write('\n'.join(generator.warnings))

    @staticmethod
    def _get_list_from_string(options):
        """
        Transforms comma separated string into a list of those elements.
        Transforms strings to ints if they are numbers.
        Eg.:
            "200,'400','xxx'" => [200, 400, 'xxx']
        """
        if options:
            return [
                int(option) if option.isdigit()
                else option.strip('/')
                for option in options.split(',')
            ]
        return None



import uuid

from django.core.management import call_command
from django.conf import settings
from django.utils.regex_helper import normalize

try:
    from django.urls import URLResolver
except ImportError:
    # Django < 2.0
    try:
        from django.urls import RegexURLResolver as URLResolver
    except ImportError:
        # Django < 1.10
        from django.core.urlresolvers import RegexURLResolver as URLResolver
from unittest import skip

from .tests import SmokeTests


def get_pattern(url_pattern):
    if hasattr(url_pattern, 'regex'):  # dj < 2.0
        return url_pattern.regex.pattern
    else:                              # dj >= 2.0
        return str(url_pattern.pattern)


class HTTPMethodNotSupported(Exception):
    pass


class UrlStructureNotSupported(Exception):
    pass


class AppNotInInstalledApps(Exception):
    pass


class SmokeTestsGenerator:
    SUPPORTED_HTTP_METHODS = ['GET', 'POST', 'PUT', 'DELETE']
    ALLOWED_STATUS_CODES = [200, 201, 301, 302, 304, 405]
    DISALLOWED_STATUS_CODES = [500, 501, 502]

    def __init__(
            self, http_methods=None, allowed_status_codes=None, disallowed_status_codes=None,
            use_db=True, app_names=None, disable_migrations=False, settings_module=None,
            configuration=None, fixture_path=None
    ):
        if http_methods:
            self.validate_custom_http_methods(http_methods)
        self.methods_to_test = http_methods or self.SUPPORTED_HTTP_METHODS
        self.allowed_status_codes = allowed_status_codes
        self.disallowed_status_codes = disallowed_status_codes or self.DISALLOWED_STATUS_CODES
        self.use_db = use_db
        self.app_names = self.validate_app_names(app_names)
        self.disable_migrations = disable_migrations
        self.settings_module = settings_module
        self.configuration = configuration
        self.fixture_path = fixture_path
        self.warnings = []

        self.all_patterns = []  # [(url_pattern, lookup_str, url_name),]

    def validate_custom_http_methods(self, http_methods):
        unsupported_methods = set(http_methods) - set(self.SUPPORTED_HTTP_METHODS)
        if unsupported_methods:
            raise HTTPMethodNotSupported(
                'Methods {} are not supported'.format(list(unsupported_methods))
            )

    @staticmethod
    def validate_app_names(app_names):
        for app_name in app_names or []:
            if app_name and app_name not in settings.INSTALLED_APPS:
                raise AppNotInInstalledApps(app_name)
        return app_names

    def _generate_test(self, url, method, detail_url=False):
        def test(self_of_test):
            http_method_function = getattr(self_of_test.client, method.lower(), None)
            response = http_method_function(url, {})
            additional_status_codes = [404] if detail_url else []
            if self.allowed_status_codes and (
                response.status_code not in self.allowed_status_codes + additional_status_codes
            ):
                self_of_test.fail_test(url, method, response=response)
            elif not self.allowed_status_codes and (
                response.status_code in self.disallowed_status_codes
            ):
                self_of_test.fail_test(url, method, response=response)
        return test

    @staticmethod
    def _generate_skipped_test():
        @skip('Not supported')
        def test(self_of_test):
            pass

        return test

    def execute(self):
        self.load_all_endpoints(URLResolver(r'^/', settings.ROOT_URLCONF).url_patterns)
        for url_pattern, lookup_str, url_name in self.all_patterns:
            if not self.app_names or self.is_url_inside_specified_app(lookup_str):
                self.create_tests_for_endpoint(url_pattern, url_name)

        if self.disable_migrations:
            self._disable_native_migrations()

        self._set_fixture_path()

        call_command_kwargs = self._get_call_command_kwargs()
        call_command('test', 'django_smoke_tests', **call_command_kwargs)

    @staticmethod
    def _disable_native_migrations():
        from .migrations import DisableMigrations
        settings.MIGRATION_MODULES = DisableMigrations()

    def _set_fixture_path(self):
        setattr(SmokeTests, 'fixture_path', self.fixture_path)

    def _get_call_command_kwargs(self):
        kwargs = {}

        if not self.use_db:
            kwargs['testrunner'] = 'django_smoke_tests.runners.NoDbTestRunner'

        if self.settings_module:
            kwargs['settings'] = self.settings_module

        if self.configuration:
            kwargs['configuration'] = self.configuration

        return kwargs

    def is_url_inside_specified_app(self, lookup_str):
        for app_name in self.app_names:
            if lookup_str.startswith(app_name):
                return True
        return False

    def load_all_endpoints(self, url_list, parent_url=''):
        for url_pattern in url_list:
            if hasattr(url_pattern, 'url_patterns'):
                self.load_all_endpoints(
                    url_pattern.url_patterns, parent_url + get_pattern(url_pattern)
                )
            else:
                self.all_patterns.append((
                    parent_url + get_pattern(url_pattern),
                    self.get_lookup_str(url_pattern),
                    url_pattern.name
                ))

    @staticmethod
    def get_lookup_str(url_pattern):
        try:
            return url_pattern.lookup_str
        except AttributeError:
            # Django < 1.10
            callback = url_pattern.callback
            if not hasattr(callback, '__name__'):
                return callback.__module__ + "." + callback.__class__.__name__
            return callback.__module__ + "." + callback.__name__

    def create_tests_for_endpoint(self, url_pattern, url_name):
        if self.is_endpoint_skipped(url_name):
            self.create_tests_for_http_methods(None, url_pattern, skipped=True)
        else:
            try:
                url_as_str, url_params = self.normalize_url_pattern(url_pattern)
            except UrlStructureNotSupported:
                self.warnings.append(
                    'Test skipped. URL << {} >> could not be parsed.'.format(
                        url_pattern
                    ))
                self.create_tests_for_http_methods(None, url_pattern, skipped=True)
            else:
                fake_params = {param: self.create_random_value() for param in url_params}
                url = self.create_url(url_as_str, fake_params)
                self.create_tests_for_http_methods(url, url_pattern, detail_url=bool(url_params))

    @staticmethod
    def is_endpoint_skipped(url_name):
        try:
            return url_name and url_name in settings.SKIP_SMOKE_TESTS
        except AttributeError:
            return False

    @staticmethod
    def normalize_url_pattern(url_pattern):
        normalized = normalize(url_pattern)

        try:
            [(url_as_str, url_params)] = normalized
        except ValueError:
            try:
                [(url_as_str_without_param, _), (url_as_str, url_params)] = normalized
            except ValueError:
                raise UrlStructureNotSupported

        if 'format' in url_params and url_as_str.endswith('.%(format)s'):
            # remove optional parameter provided by DRF ViewSet
            # eg. /items is provided as /items.%(format)s (/items.json)
            url_params.remove('format')
            url_as_str = url_as_str[:-len('.%(format)s')]

        return url_as_str, url_params

    @staticmethod
    def create_random_value():
        return uuid.uuid4()

    @staticmethod
    def create_url(url_as_str, parameters):
        url = url_as_str % parameters
        return url if url.startswith('/') else '/{}'.format(url)

    def create_tests_for_http_methods(self, url, url_pattern, detail_url=False, skipped=False):
        for method in self.methods_to_test:
            self.create_test_for_http_method(method, url, url_pattern, detail_url, skipped)

    def create_test_for_http_method(
            self, method, url, url_pattern=None, detail_url=False, skipped=False
    ):
        if skipped:
            test = self._generate_skipped_test()
        else:
            test = self._generate_test(url, method, detail_url)

        if not url_pattern:
            url_pattern = url  # url and url_pattern are the same when there are no URL parameters
        setattr(SmokeTests, self.create_test_name(method, url_pattern), test)

    @staticmethod
    def create_test_name(method, url_pattern):
        return 'test_smoke_{}_{}'.format(method, url_pattern)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import re
import sys

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


def get_version(*file_paths):
    """Retrieves the version from django_smoke_tests/__init__.py"""
    filename = os.path.join(os.path.dirname(__file__), *file_paths)
    version_file = open(filename).read()
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError('Unable to find version string.')


version = get_version("django_smoke_tests", "__init__.py")


if sys.argv[-1] == 'publish':
    try:
        import wheel
        print("Wheel version: ", wheel.__version__)
    except ImportError:
        print('Wheel library missing. Please run "pip install wheel"')
        sys.exit()
    os.system('python setup.py sdist upload')
    os.system('python setup.py bdist_wheel upload')
    sys.exit()

if sys.argv[-1] == 'tag':
    print("Tagging the version on git:")
    os.system("git tag -a %s -m 'version %s'" % (version, version))
    os.system("git push --tags")
    sys.exit()

readme = open('README.rst').read()

_read = lambda f: open(
    os.path.join(os.path.dirname(__file__), f)).read() if os.path.exists(f) else ''

install_requires = [
    l.replace('==', '>=') for l in _read('requirements.txt').split('\n')
    if l and not l.startswith('#') and not l.startswith('-')]

setup(
    name='django-smoke-tests',
    version=version,
    description="""Automatic smoke tests for Django project.""",
    long_description=readme,
    author='Kamil Kijak',
    author_email='kamilkijak@gmail.com',
    url='https://github.com/kamilkijak/django-smoke-tests',
    packages=[
        'django_smoke_tests',
    ],
    include_package_data=True,
    install_requires=install_requires,
    license="MIT",
    zip_safe=False,
    keywords=['django-smoke-tests', 'test', 'smoke'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.11',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
)



from django.test.runner import DiscoverRunner


class NoDbTestRunner(DiscoverRunner):
    """ A test runner to test without database creation """

    def setup_databases(self, **kwargs):
        pass

    def teardown_databases(self, old_config, **kwargs):
        pass
